﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Xml;
using BlogEngine.Core;
using BlogEngine.Core.Web.Controls;

namespace Prabir.BlogEngine.Extensions
{
    [Extension("Comment Exporter", "0.1", "<a href='http://www.prabir.me' target='_blank'>Prabir Shrestha</a>")]
    public class CommentExporter
    {
        private const string EXTENSION_NAME = "CommentExporter";
        private const string EXTENSION_ADMIN_PAGE = "~/User controls/ExportComments.aspx";

        public CommentExporter()
        {
            ExtensionManager.SetAdminPage(EXTENSION_NAME, EXTENSION_ADMIN_PAGE);
        }
    }

    public interface ICommentExporter<T>
    {
        T Export();
        Dictionary<string, string> AuthorsMap { get; set; }
    }

    public class DisqusCommentExporter : ICommentExporter<XmlDocument>
    {
        public XmlDocument Export()
        {
            XmlDocument xDoc = new XmlDocument();
            // XML declaration
            XmlNode declaration = xDoc.CreateNode(XmlNodeType.XmlDeclaration, null, null);
            xDoc.AppendChild(declaration);

            // Root element: output
            XmlElement root = xDoc.CreateElement("articles");
            xDoc.AppendChild(root);

            foreach (Post post in Post.Posts)
            {
                if (post.Comments.Count == 0)
                    continue;

                // Sub-element: article
                XmlElement article = xDoc.CreateElement("article");
                root.AppendChild(article);

                #region Article

                // Sub-element: url
                XmlElement url = xDoc.CreateElement("url");
                url.InnerText = post.AbsoluteLink.ToString();
                article.AppendChild(url);

                #region Comments

                // Sub-element: comments
                XmlElement comments = xDoc.CreateElement("comments");
                article.AppendChild(comments);

                foreach (Comment comment in post.Comments)
                {
                    if (comment.Email == "pingback" || comment.Email == "trackback")
                        continue;

                    XmlElement commentRoot = xDoc.CreateElement("comment");
                    comments.AppendChild(commentRoot);

                    bool isAnnonymous = true;

                    if (AuthorsMap.ContainsKey(comment.Author))
                        isAnnonymous = false;

                    XmlElement name = xDoc.CreateElement("name");
                    name.InnerText = isAnnonymous ? comment.Author : AuthorsMap[comment.Author];
                    commentRoot.AppendChild(name);

                    XmlElement email = xDoc.CreateElement("email");
                    email.InnerText = comment.Email;
                    commentRoot.AppendChild(email);

                    // Sub-element: url
                    XmlElement website = xDoc.CreateElement("url");
                    if (comment.Website != null)
                        website.InnerText = comment.Website.ToString();
                    commentRoot.AppendChild(website);

                    XmlElement ip = xDoc.CreateElement("ip_address");
                    ip.InnerText = comment.IP;
                    commentRoot.AppendChild(ip);

                    XmlElement message = xDoc.CreateElement("message");
                    message.InnerText = comment.Content;
                    commentRoot.AppendChild(message);

                    // todo: fix date and gmt
                    XmlElement date = xDoc.CreateElement("date");
                    date.InnerText = comment.DateCreated.ToString("ddd, dd MMM yyyy HH:mm:ss -0000");
                    commentRoot.AppendChild(date);

                    XmlElement points = xDoc.CreateElement("points");
                    points.InnerText = "1";
                    commentRoot.AppendChild(points);

                }

                #endregion

                #endregion

            }

            return xDoc;
        }

        private Dictionary<string, string> _authorsMap = new Dictionary<string, string>();
        public Dictionary<string, string> AuthorsMap
        {
            get { return _authorsMap; }
            set { _authorsMap = value; }
        }
    }

    public class IntenseDebateCommentExporter : ICommentExporter<XmlDocument>
    {
        public XmlDocument Export()
        {
            XmlDocument xDoc = new XmlDocument();
            // XML declaration
            XmlNode declaration = xDoc.CreateNode(XmlNodeType.XmlDeclaration, null, null);
            xDoc.AppendChild(declaration);

            // Root element: output
            XmlElement root = xDoc.CreateElement("output");
            xDoc.AppendChild(root);

            foreach (Post post in Post.Posts)
            {
                if (post.Comments.Count == 0)
                    continue;

                // Sub-element: blogpost
                XmlElement blogpost = xDoc.CreateElement("blogpost");
                root.AppendChild(blogpost);

                // Sub-element: url
                XmlElement url = xDoc.CreateElement("url");
                url.InnerText = HttpUtility.UrlEncode(post.AbsoluteLink.ToString());
                blogpost.AppendChild(url);

                // Sub-element: title
                XmlElement title = xDoc.CreateElement("title");
                title.InnerText = post.Title;
                blogpost.AppendChild(title);

                // Sub-element: guid
                XmlElement guid = xDoc.CreateElement("guid");
                guid.InnerText = post.PermaLink.ToString();
                blogpost.AppendChild(guid);

                #region Comments

                // comments
                XmlElement comments = xDoc.CreateElement("comments");
                blogpost.AppendChild(comments);

                foreach (Comment comment in post.Comments)
                {

                    XmlElement commentRoot = xDoc.CreateElement("comment");
                    comments.AppendChild(commentRoot);

                    bool isAnnonymous = true;

                    if (AuthorsMap.ContainsKey(comment.Author))
                        isAnnonymous = false;

                    // Sub-element: isAnon
                    XmlElement isAnon = xDoc.CreateElement("isAnon");
                    isAnon.InnerText = isAnnonymous ? "1" : "0";
                    commentRoot.AppendChild(isAnon);

                    // Sub-element: name
                    XmlElement name = xDoc.CreateElement("name");
                    name.InnerText = isAnnonymous ? comment.Author : AuthorsMap[comment.Author];
                    commentRoot.AppendChild(name);

                    // Sub-element: email
                    XmlElement email = xDoc.CreateElement("email");
                    email.InnerText = comment.Email;
                    commentRoot.AppendChild(email);

                    // Sub-element: url
                    XmlElement website = xDoc.CreateElement("url");
                    if (comment.Website != null)
                        website.InnerText = comment.Website.ToString();
                    commentRoot.AppendChild(website);

                    // Sub-element: ip
                    XmlElement ip = xDoc.CreateElement("ip");
                    ip.InnerText = IPAddressToNumber(comment.IP).ToString();
                    commentRoot.AppendChild(ip);

                    // Sub-element: comment
                    XmlElement commentContent = xDoc.CreateElement("comment");
                    commentContent.InnerText = comment.Content;
                    commentRoot.AppendChild(commentContent);

                    // todo: fix date and gmt
                    // Sub-element: date
                    XmlElement date = xDoc.CreateElement("date");
                    date.InnerText = comment.DateCreated.ToString("yyyy-MM-dd HH:mm:ss");
                    commentRoot.AppendChild(date);

                    // Sub-element: gmt
                    XmlElement gmt = xDoc.CreateElement("gmt");
                    gmt.InnerText = comment.DateCreated.ToString("yyyy-MM-dd HH:mm:ss");
                    commentRoot.AppendChild(gmt);

                    // Sub-element: score
                    XmlElement score = xDoc.CreateElement("score");
                    score.InnerText = isAnnonymous ? "0" : "1";
                    commentRoot.AppendChild(score);

                }

                #endregion

            }

            return xDoc;
        }

        private Dictionary<string, string> _authorsMap = new Dictionary<string, string>();
        public Dictionary<string, string> AuthorsMap
        {
            get { return _authorsMap; }
            set { _authorsMap = value; }
        }

        public static double IPAddressToNumber(string IPaddress)
        {
            int i;
            string[] arrDec;
            double num = 0;
            if (IPaddress == "")
            {
                return 0;
            }
            else
            {
                arrDec = IPaddress.Split('.');
                for (i = arrDec.Length - 1; i >= 0; i--)
                {
                    num += ((int.Parse(arrDec[i]) % 256) * Math.Pow(256, (3 - i)));
                }
                return num;
            }
        }
    }
}